package com.jiuqi.util;

import java.util.ArrayList;
import java.util.List;

public class MemStream extends Stream
{
    private static final int PAGE_SIZE = 1024;
    private List pages;
    private long size;
    private int curPageNo;
    private int curPos;
    
    private byte[] curPage() {
        return (byte[])((this.curPageNo < this.pages.size()) ? ((byte[])this.pages.get(this.curPageNo)) : null);
    }
    
    public MemStream() {
        this.pages = new ArrayList();
        this.size = 0L;
        this.curPageNo = 0;
        this.curPos = 0;
    }
    
    @Override
    public void setSize(final long newSize) throws StreamException {
        if (newSize < 0L) {
            throw new StreamException("Stream size error");
        }
        this.size = newSize;
        int pagecount = (int)(this.size / 1024L);
        if (this.size % 1024L > 0L) {
            ++pagecount;
        }
        if (pagecount > this.pages.size()) {
            for (int k = pagecount - this.pages.size(), i = 0; i < k; ++i) {
                this.pages.add(new byte[1024]);
            }
        }
        else {
            for (int k = this.pages.size() - pagecount, i = 0; i < k; ++i) {
                this.pages.remove(this.pages.size() - 1);
            }
        }
        if (this.curPageNo * 1024 + this.curPos > this.size) {
            this.curPageNo = (int)(this.size / 1024L);
            this.curPos = (int)(this.size % 1024L);
        }
    }
    
    @Override
    public int read(final byte[] buffer, int pos, int count) throws StreamException {
        if (pos < 0 || count < 0) {
            throw new StreamException("Stream read error");
        }
        if (count == 0 || pos >= buffer.length) {
            return 0;
        }
        if (pos + count >= buffer.length) {
            count = buffer.length - pos;
        }
        if (this.size - this.curPageNo * 1024 - this.curPos < count) {
            count = (int)(this.size - this.curPageNo * 1024 - this.curPos);
        }
        int readed = 0;
        while (readed < count) {
            final byte[] page = this.curPage();
            int canread = count - readed;
            if (canread > 1024 - this.curPos) {
                canread = 1024 - this.curPos;
            }
            System.arraycopy(page, this.curPos, buffer, pos, canread);
            pos += canread;
            readed += canread;
            this.curPos += canread;
            if (this.curPos == 1024) {
                ++this.curPageNo;
                this.curPos = 0;
            }
        }
        return count;
    }
    
    @Override
    public byte read() throws StreamException {
        final byte[] page = this.curPage();
        if (page != null) {
            final byte rt = page[this.curPos];
            ++this.curPos;
            if (this.curPos == 1024) {
                this.curPos = 0;
                ++this.curPageNo;
            }
            return rt;
        }
        throw new StreamException("Stream read error");
    }
    
    @Override
    public int write(final byte[] buffer, int pos, int count) throws StreamException {
        if (pos < 0 || count < 0) {
            throw new StreamException("Stream read error");
        }
        if (count == 0 || buffer == null || pos >= buffer.length) {
            return 0;
        }
        if (pos + count >= buffer.length) {
            count = buffer.length - pos;
        }
        long p = this.getPosition();
        int writed = 0;
        while (writed < count) {
            byte[] page = this.curPage();
            if (page == null) {
                page = new byte[1024];
                this.pages.add(page);
            }
            int canwrite = 1024 - this.curPos;
            if (canwrite > count - writed) {
                canwrite = count - writed;
            }
            System.arraycopy(buffer, pos, page, this.curPos, canwrite);
            writed += canwrite;
            p += canwrite;
            if (p > this.size) {
                this.size = p;
            }
            this.curPos += canwrite;
            pos += canwrite;
            if (this.curPos == 1024) {
                this.curPos = 0;
                ++this.curPageNo;
            }
        }
        return count;
    }
    
    @Override
    public void write(final byte value) throws StreamException {
        byte[] page = this.curPage();
        if (page == null) {
            page = new byte[1024];
            this.pages.add(page);
        }
        page[this.curPos] = value;
        ++this.curPos;
        if (this.curPos >= 1024) {
            this.curPos = 0;
            ++this.curPageNo;
        }
        if (this.curPageNo * 1024 + this.curPos > this.size) {
            ++this.size;
        }
    }
    
    @Override
    public long seek(final long offset, final int origin) throws StreamException {
        switch (origin) {
            case 0: {
                if (offset < 0L || offset > this.size) {
                    throw new StreamException("Stream seek error");
                }
                this.curPageNo = (int)(offset / 1024L);
                this.curPos = (int)(offset % 1024L);
                break;
            }
            case 1: {
                final long x = offset + this.curPageNo * 1024 + this.curPos;
                if (x < 0L || x > this.size) {
                    throw new StreamException("Stream seek error");
                }
                this.curPageNo = (int)(x / 1024L);
                this.curPos = (int)(x % 1024L);
                break;
            }
            case 2: {
                if (offset > 0L || offset + this.size < 0L) {
                    throw new StreamException("Stream seek error");
                }
                final long x = offset + this.size;
                this.curPageNo = (int)(x / 1024L);
                this.curPos = (int)(x % 1024L);
                break;
            }
        }
        return this.curPageNo * 1024 + this.curPos;
    }
}
